;;; ralee-tools

; Copyright (c) 2004 Sam Griffiths-Jones
;
; This is part of RALEE -- see
; http://www.sanger.ac.uk/Users/sgj/ralee/ and the 00README file
; that should accompany this file.
;
; RALEE is free software; you can redistribute it and/or modify it
; under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; RALEE is distributed in the hope that it will be useful, but
; WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
; General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with RALEE; if not, write to the Free Software Foundation,
; Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


(defvar ralee-seqid-regex "[A-Za-z0-9/_,.-]+")
(defvar ralee-nse-regex (concat ralee-seqid-regex "/[0-9]+-[0-9]+"))
(defvar ralee-seq-regex "[A-Za-z.-]+")


(defun current-line ()  ; surely this should be a default method?
  "Return the vertical position of point..."
  (count-lines (point-min) (point)))


(defun ralee-is-alignment-line ()
  "Check if the current line is part of the alignment itself"
  (save-excursion
    (beginning-of-line)
    (looking-at (concat ralee-seqid-regex "\\( \\|\t\\)+" ralee-seq-regex "$"))
  ))

(defun ralee-is-markup-line ()
  "Check if the current line is #=G[CR]"
  (save-excursion
    (beginning-of-line)
    (looking-at "#=G[CR] ")
  ))


(defun ralee-is-alignment-column ()
  "Check if the current column is inside the alignment"
  (save-excursion
    (let ((column (current-column)))
      (ralee-find-first-line)
      (move-to-column column)
      (if (or (looking-at (concat ralee-seq-regex "$"))
	      (looking-at "$"))
	  t
	)
      )
    )
  )


(defun ralee-find-first-line ()
  "move to the first alignment or markup line"
  (goto-char (point-min))
  (while (and (equal (ralee-is-alignment-line) nil)
	      (equal (ralee-is-markup-line) nil))
    (forward-line)
    )
  (current-line)
  )


(defun ralee-find-first-column ()
  "move to the first column in the alignment"
  (if (or (ralee-is-alignment-line) (ralee-is-markup-line))
      (progn
	(end-of-line)
	(re-search-backward (concat " " ralee-seq-regex))
	(forward-char)
	(current-column)
	)
    )
  )


(defun ralee-get-seq-id ()
  "get the sequence identifier of the current alignment line"
  (save-excursion
    (beginning-of-line)
    (let ((beg (point)))
      (if (ralee-is-alignment-line)
	  (progn
	    (looking-at ralee-seqid-regex)
	    (match-string 0)
	    )
	(if (ralee-is-markup-line)
	    (if (looking-at (concat "#=GR " ralee-seqid-regex ralee-seqid-regex))
		(match-string 0)
	      (if (looking-at (concat "#=GC " ralee-seqid-regex))
		  (match-string 0)
		)
	      )
	  (progn
	    (message "can't get seqid from current line")
	    nil
	    )
	  )
	)
      )
    )
  )


(defun ralee-get-name-start-end ()
  "get the name/st-en from the current alignment line"
  (save-excursion
    (let (name
	  (tag nil)
	  (st nil)
	  (en nil))

      (beginning-of-line)
      (if (or (ralee-is-alignment-line) (ralee-is-markup-line))
	  (progn 
	    (if (looking-at (concat "#=GR " ralee-seqid-regex " \\(" ralee-seqid-regex "\\)"))
		(setq tag (match-string 1))
	      )

	    (if (looking-at (concat "\\(\\(?:#=G[RC] \\)?" ralee-seqid-regex "\\)/\\([0-9]+\\)-\\([0-9]+\\)"))
		(progn
		  (setq name (match-string 1))
		  (setq st (string-to-number (match-string 2)))
		  (setq en (string-to-number (match-string 3)))
		  )
	      (if (looking-at (concat "\\(?:#=G[RC] \\)?" ralee-seqid-regex))
		  (progn
		    (setq name (match-string 0))
		    ;; can use the below to make up st/en
		    ;; but we don't want to at the moment
		    ;;(setq st 1)
		    ;;(setq en (length 
		    ;;	    (split-string 
		    ;; 	     (ralee-ungap-string(ralee-get-seq-string)) "")))
		    )
		)
	      )
	    )
	)
      (if name
	  (list name st en tag)
	)
      )
    )
  )


(defun ralee-get-seq-string ()
  "get the sequence string of the current alignment line"
  (save-excursion
    (if (or (ralee-is-alignment-line) (ralee-is-markup-line))
	(progn
	  (end-of-line)
	  (let ((end (point)))
	    (search-backward " ")
	    (copy-region-as-kill (1+ (point)) end)
	    (car kill-ring)
	    )
	  )
      )
    )
  )


(defun ralee-ungap-string (string)
  "take a seq string as input and return an ungapped version"
  (let (split
	base
	(out ""))
    (setq split (split-string string ""))
    (while split
      (setq base (car split))
      (setq split (cdr split))
      (if (string-match "[A-Za-z]" base)
	  (setq out (concat out base))
	)
      )
    out
    )
  )
  

(defun ralee-maxidlength ()
  "get the maximum id length for the current alignment"
  (interactive)
  (save-excursion
    (goto-char (point-min))
    (let ((maxidlength 0)
	  seqid
	  split)
      (while (< (point) (point-max))
	(beginning-of-line)
	(if (or (ralee-is-alignment-line) (ralee-is-markup-line))
	    (progn
	      (setq seqid (ralee-get-seq-id))
	      (setq splitlen (length (split-string seqid "")))
	      
	      ;; Grrrr
	      (if (featurep 'xemacs)
		  (setq splitlen (- splitlen 2))
		)

	      (if (> splitlen maxidlength)
		  (setq maxidlength splitlen)
		)
	      )
	  )
	(forward-line)
	)
      maxidlength
      )
    )
  )


(defun ralee-count-bases-in-column ()
  "return the counts of bases in the current column"
  (save-excursion
    (let (column
	  (a-count 0)
	  (c-count 0)
	  (g-count 0)
	  (u-count 0)
	  (line-count 0)
	  )

      (setq column (current-column))
      (goto-char (point-min))

      (while (< (point) (point-max))
	(if (ralee-is-alignment-line)
	    (progn
	      (move-to-column column)
	      (setq line-count (1+ line-count))
	      (let ((base (char-after))
		    (case-fold-search t))
		(if (char-equal base ?A) (setq a-count (1+ a-count)))
		(if (char-equal base ?C) (setq c-count (1+ c-count)))
		(if (char-equal base ?G) (setq g-count (1+ g-count)))
		(if (or (char-equal base ?U)
			(char-equal base ?T)) (setq u-count (1+ u-count)))
		)
	      )
	  )
	(forward-line)
	)
      (list line-count a-count c-count g-count u-count)
      )
    )
  )


(defun ralee-blocked-alignment-p ()
  "check if surrent alignment is blocked"
  (save-excursion
    (ralee-find-first-line)
    (let ((id (ralee-get-seq-id)))
      (forward-line)
      (search-forward id nil t)
      )
    )
  )


(provide 'ralee-tools)
